/*
  IeeFlip Library - Implementation

  Originally created 5 March 2011

  This file is in the public domain.  
*/

#include "IeeFLip.h"
#include "WProgram.h"

IeeFlip::IeeFlip(uint8_t csPin, uint8_t wrPin, uint8_t rdPin, uint8_t address0Pin,
                 uint8_t data0Pin, uint8_t data1Pin, uint8_t data2Pin, uint8_t data3Pin,
                 uint8_t data4Pin, uint8_t data5Pin, uint8_t data6Pin, uint8_t data7Pin)
{
  // Copy pin values.
  _csPin = csPin;
  _wrPin = wrPin;
  _rdPin = rdPin;
  _address0Pin = address0Pin;
  
  _dataPins[0] = data0Pin;
  _dataPins[1] = data1Pin;
  _dataPins[2] = data2Pin;
  _dataPins[3] = data3Pin;
  _dataPins[4] = data4Pin;
  _dataPins[5] = data5Pin;
  _dataPins[6] = data6Pin;
  _dataPins[7] = data7Pin;

  // Init screen character dimensions to something harmless.
  _numDisplayColumns = 1;
  _numDisplayRows = 1;
  _numDisplayCharacters = _numDisplayColumns * _numDisplayRows;

  // Initialize pin states. Everything to input, no pullup register.
  pinMode(csPin, INPUT);
  digitalWrite(csPin, LOW);

  pinMode(wrPin, INPUT);
  digitalWrite(wrPin, LOW);

  if (rdPin != DO_NOT_USE_PIN)
  {
    pinMode(rdPin, INPUT);
    digitalWrite(rdPin, LOW);
  }
  
  if (address0Pin != DO_NOT_USE_PIN)
  {
    pinMode(address0Pin, INPUT);
    digitalWrite(address0Pin, LOW);
  }
  
  for (int i=0; i<NUM_DATA_PINS; i++)
  {
    uint8_t pin = _dataPins[i];
    
    if (pin != DO_NOT_USE_PIN)
    {
      pinMode(pin, INPUT);
      digitalWrite(pin, LOW);
    }
  }
}

void IeeFlip::begin(uint8_t numColumns, int numRows)
{
  _numDisplayColumns = numColumns;
  _numDisplayRows = numRows;
  _numDisplayCharacters = numColumns * numRows;

  // Initialize VFD to match default power on settings.
  // This is necessary since the last sketch run may have changed something.
  clear();

#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
	
  controlCharacter(END_BLINKING_CHARS_CHARACTER);
  timingDelay(BLINK_SET_WAIT_TIME_IN_MICROSECONDS);
  controlCharacter(BLINK_CURSOR_OFF_CHARACTER);
  controlCharacter(END_OF_LINE_WRAP_CHARATCER);

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

// To experiment with a newer display, comment out this line:
#error Need to figure out how to set newer displays to power on state.
	
#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}

void IeeFlip::clear()
{
#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  if (_numDisplayRows <= 1)
  {
    // Line feed, carriage return does the trick.
    controlCharacter(LINE_FEED_CHARACTER);
    timingDelay(VSCROLL_WAIT_TIME_IN_MICROSECONDS);
    controlCharacter(CARRIAGE_RETURN_CHARACTER);  
  }
  else
  {
    // Not even sure if such a display exists, but this should work.
    for (uint8_t row=0; row<_numDisplayRows; row++)
    {
      setCursor(0, row);
      for (uint8_t count=_numDisplayCharacters; count; count--)
      {
        write(' ');
      }
    }
    // Home cursor.
    setCursor(0, 0);
  }

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(CLEAR_DISPLAY_HOME_CURSOR_CHARACTER);
  timingDelay(CLEAR_WAIT_TIME_IN_MICROSECONDS);

#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}

void IeeFlip::home()
{
#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  if (_numDisplayRows <= 1)
  {
    // Carriage return does the trick.
    controlCharacter(CARRIAGE_RETURN_CHARACTER);  
  }
  else
  {
    // Not even sure if such a display exists, but this should work.
    setCursor(0, 0);
  }

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(HOME_CURSOR_CHARACTER);
  timingDelay(HOME_WAIT_TIME_IN_MICROSECONDS);

#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}
  
#if !IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

void IeeFlip::noDisplay()
{
  controlCharacter(BLANK_DISPLAY_CHARACTER);
}

void IeeFlip::display()
{
  controlCharacter(UNBLANK_DISPLAY_CHARACTER);
}

#endif // !IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

void IeeFlip::noBlink()
{
#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(BLINK_CURSOR_OFF_CHARACTER);

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(CURSOR_OFF_CHARACTER);

#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}

void IeeFlip::blink()
{
#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(BLINK_CURSOR_ON_CHARACTER);

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(BLINK_BLOCK_CURSOR_CHARACTER);
  timingDelay(BLINK_SET_WAIT_TIME_IN_MICROSECONDS);
  controlCharacter(CURSOR_ON_CHARACTER);

#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}

void IeeFlip::noCursor()
{
#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  noBlink(); // Old display doesn't support an underline character.

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(CURSOR_OFF_CHARACTER);

#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}

void IeeFlip::cursor()
{
#if IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  blink(); // Old display doesn't support an underline character.

#else // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

  controlCharacter(UNDERBAR_CURSOR_CHARACTER);
  controlCharacter(CURSOR_ON_CHARACTER);

#endif // IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
}

void IeeFlip::setCursor(uint8_t column, uint8_t row)
{
  uint8_t address = row * _numDisplayColumns + column;
  if (address >= _numDisplayCharacters)
  {
    address = 0;
  }
  
  controlCharacter(CURSOR_SET_CHARACTER);
  writeInternal(address, LOW);
  
  timingDelay(CURSOR_SET_WAIT_TIME_IN_MICROSECONDS);
}

/*virtual*/ void IeeFlip::write(uint8_t character)
{
  // Remap from LCD custom character locations to Flip locations.
  writeInternal(character, LOW);  
}

void IeeFlip::controlCharacter(uint8_t character)
{
  writeInternal(character, (character > FIRST_HIGH_CONTROL_CHARACTER) ? HIGH : LOW);
}

#if !IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY // Can't find any read commands that work with the old display.

uint8_t IeeFlip::read(uint8_t address0Level)
{
  if (_rdPin==DO_NOT_USE_PIN)
  {
    // Can't set the /RD pin so can't 
    // read from the display.
    return 0xFF;
  }
  
  // Make sure control pins are high. The diplay pullups should
  // be pulling them high. If not, it probably means the display is not
  // powered up in which case we don't want to start driving it as
  // we might damage it.
  if (digitalRead(_csPin)==LOW || digitalRead(_wrPin)==LOW || digitalRead(_rdPin)==LOW)
  {
    // Bail, display may not be powered up.
    return 0xFF;
  }
  
  // Setup A0.
  if (_address0Pin!=DO_NOT_USE_PIN)
  {
    digitalWrite(_address0Pin, address0Level);
    pinMode(_address0Pin, OUTPUT);
  }

  // Pull down /CS.
  digitalWrite(_csPin, LOW);
  pinMode(_csPin, OUTPUT);
  
  // Pull down /RD. This puts data on the data pins.
  digitalWrite(_rdPin, LOW);
  pinMode(_rdPin, OUTPUT);
  
  // Read data from data pins.
  // Pin mode is already set to floating inputs.
  uint8_t result = 0;
  for (int i=0; i<NUM_DATA_PINS; i++)
  {
    uint8_t pin = _dataPins[i];
    if (pin != DO_NOT_USE_PIN)
    {
      // Get pin value.
      if (digitalRead(pin)==HIGH)
      {
        // Pin is set, or that into the result.
        result |= (1<<i);
      }
    }
  }  

  // Pull /RD high.
  digitalWrite(_rdPin, HIGH);
  
  // Pull /CS high.
  digitalWrite(_csPin, HIGH);
    
  // Restore address 0 to open circuit.
  if (_address0Pin != DO_NOT_USE_PIN)
  {
    // Pull high first.
    digitalWrite(_address0Pin, HIGH);
    
    // Disable pin output and pullup. The display pullup resistors will keep it high.
    pinMode(_address0Pin, INPUT);
    digitalWrite(_address0Pin, LOW);
  }
  
  // Disable pin output and pullup for /CS and /RD.
  // The display pullup resisitors will keep them high.
  pinMode(_rdPin, INPUT);
  digitalWrite(_rdPin, LOW);

  pinMode(_csPin, INPUT);
  digitalWrite(_csPin, LOW);
  
  timingDelay(READ_CHARACTER_WAIT_TIME_IN_MICROSECONDS);

  return result;
}

#endif // !IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY

void IeeFlip::writeInternal(uint8_t character, uint8_t address0Level)
{
  if (_address0Pin==DO_NOT_USE_PIN && address0Level==HIGH)
  {
    // Can't set the address pin high so bail.
    return;
  }

  if (_dataPins[7]==DO_NOT_USE_PIN && (character & 0b10000000))
  {
    // Can't set the high data bit so bail.
    return;
  }
  
  // Make sure control pins are high. The diplay pullups should
  // be pulling them high. If not, it probably means the display is not
  // powered up in which case we don't want to start driving it as
  // we might damage it.
  if (digitalRead(_csPin)==LOW || digitalRead(_wrPin)==LOW)
  {
    // Bail, display may not be powered up.
    return;
  }
    
  // Setup A0.
  if (_address0Pin!=DO_NOT_USE_PIN)
  {
    digitalWrite(_address0Pin, address0Level);
    pinMode(_address0Pin, OUTPUT);
  }
  
  // Pull down /CS.
  digitalWrite(_csPin, LOW);
  pinMode(_csPin, OUTPUT);
  
  // Pull down /WR.
  digitalWrite(_wrPin, LOW);
  pinMode(_wrPin, OUTPUT);
  
  // Place character onto data pins.
  uint8_t tmpCharacter = character;
  for (int i=0; i<NUM_DATA_PINS; i++)
  {
    uint8_t pin = _dataPins[i];
    if (pin != DO_NOT_USE_PIN)
    {
      // Set pin value.
      digitalWrite(pin, (tmpCharacter & 1) ? HIGH : LOW);
          
      // Enable pin output.
      pinMode(pin, OUTPUT);  
    }
    tmpCharacter >>= 1;
  }  
  
  // Pull /WR high. This latches in the data.
  digitalWrite(_wrPin, HIGH);

  // Restore data pins to open circuit.
  for (int i=0; i<NUM_DATA_PINS; i++)
  {
    uint8_t pin = _dataPins[i];
    if (pin != DO_NOT_USE_PIN)
    {
      // Disable pin output and pullup.
      pinMode(pin, INPUT);
      digitalWrite(pin, LOW);
    }
  }

  // Pull /CS high.
  digitalWrite(_csPin, HIGH);
    
  // Restore address 0 to open circuit.
  if (_address0Pin != DO_NOT_USE_PIN)
  {
    // Pull high first.
    digitalWrite(_address0Pin, HIGH);
    
    // Disable pin output and pullup. The display pullup resistors will keep it high.
    pinMode(_address0Pin, INPUT);
    digitalWrite(_address0Pin, LOW);
  }
  
  // Disable pin output and pullup for /CS and /WR.
  // The display pullup resisitors will keep them high.
  pinMode(_wrPin, INPUT);
  digitalWrite(_wrPin, LOW);

  pinMode(_csPin, INPUT);
  digitalWrite(_csPin, LOW);
  
  timingDelay(SEND_CHARACTER_WAIT_TIME_IN_MICROSECONDS);
}



