/*
  IeeFlip Library - Serial Test
 
 Demonstrates the use an IEE Flip VFD. Tested
 with model 03700-06-020/S 1x20 alphanumeric display.
 
 This sketch displays text sent over the serial port 
 (e.g. from the Serial Monitor) on an attached LCD. Configure
 the serial monitor to 9600 baud.
 
 Use of the '~' character allows sending of hex codes:
 
 * ~x0D sends character 0D hex via write()
 * ~c0D sends character 0D hex via controlCharacter()
 * ~r08 reads 8 characters from the display and sends them
   them to the serial monitor. Values below 20 hex are displayed
   as #.
 * ~s20 displays 16 sequential characters to the display, starting
   with the character corresponding to hex 20 (space).
 * ~~ displays ~
    
  The circuit:
  
 * Display power connector (+5v & GND) connected to regulated 5v, 600ma power supply.
 * Display signal ground connected to Arduino ground.
 * Display data pins 0 through 7 connected to Arduino pins 7, 6, 5, 4, 3, 2, 14 and 15.
 * Display /CD pin connected to Ardunio pin 11
 * Display /WR pin connected to Arduino pin 8
 * Display /RD pin connected to Arduino pin 10
 * Display A0 pin connected to Arduino pin 9
 
 Created Mar 6, 2011.
 
 This example code is in the public domain.

 http://arduino.cc/playground/Main/IeeFlip
*/

// Include the library code.
#include <IeeFlip.h>

// Initialize the library with the numbers of the interface pins
IeeFlip vfd(11, 8, 10, 9, /* control pins */
            7, 6, 5, 4, 3, 2, 14, 15 /* data pins */);

uint8_t getch()
{
  while (!Serial.available()) {}

  return Serial.read();
}

// Convertex a HEX character to a number.
byte ConvertHexToNumber(char hexCharacter)
{
  if (hexCharacter>='0' && hexCharacter<='9')
  {
    return hexCharacter-'0';
  }
  else if (hexCharacter>='A' && hexCharacter<='F')
  {
    return hexCharacter-'A'+0xA;
  }
  else if (hexCharacter>='a' && hexCharacter<='f')
  {
    return hexCharacter-'a'+0xA;
  }
  else
  {
    return 0;
  }
}

void setup() {

  // set up the LCD's number of columns and rows: 
  vfd.begin(20, 1);
  
  // Print an initial message to the LCD.
  vfd.print("Serial Test.");
  
  Serial.begin(9600);
}

void loop() {

  uint8_t character = getch();
  if (character =='~')
  {
    uint8_t command = getch();
    
    switch (command)
    {
      case 'x':
      case 'X':
        // Send a character via write(). 
        {
          uint8_t character = (ConvertHexToNumber(getch()) << 4) + ConvertHexToNumber(getch());
          vfd.write(character);
          break;
        }
      case 'c':
      case 'C':
        // Send a character via controlCharacter().
        {
          uint8_t character = (ConvertHexToNumber(getch()) << 4) + ConvertHexToNumber(getch());          
          vfd.controlCharacter(character);
          break;
        }
#if !IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
      case 'r':
      case 'R':
        // Read 'n' bytes from display and send back over serial.
        {
          uint8_t readCount = (ConvertHexToNumber(getch()) << 4) + ConvertHexToNumber(getch());          
          for (int i=0; i<readCount; i++)
          {
            uint8_t result = vfd.read(LOW);
            if (result < ' ')
            {
              result = '#';
            }
            Serial.write(result);
          }
          break;
        }
#endif // !IEE_FLIP_OLD_SCHOOL_ALPHANUMERIC_DISPLAY
      case 's':
      case 'S':
        {
          // Print characters from character set, starting at character given.
          uint8_t startCharacter = (ConvertHexToNumber(getch()) << 4) + ConvertHexToNumber(getch());
          vfd.clear();
          for (int i=0; i<16; i++)
          {
            vfd.write(startCharacter + i);
          }
          break;
        }
      case '~':
        // Send tilde.
        vfd.write('~');
        break;
      default:
        // Invalid command.
        break;
    }
  }
  else
  {  
    vfd.write(character);
  }
}

